######### PET_driver.py ###########
##
## This component can be considered as an example for implementation
## of ET_PriestlyTaylor.py
##
##
##
#############################################


from ET_PriestlyTaylor import ET

""" Create an ET object """
Run = ET()

""" Initialize ET object """
Run.initialize()

""" Update ET object """
Run.update()

""" Plot ET object """
Run.plott()
